import re
import collections
import numpy as np
import pandas as pd
import graphviz as gr
from sklearn import tree
from scipy.io import arff
from sklearn.tree import DecisionTreeClassifier
from sklearn.neighbors import KNeighborsClassifier
from sklearn.preprocessing import OneHotEncoder
from sklearn.model_selection import train_test_split
from sklearn.metrics import classification_report
from sklearn.preprocessing import normalize

dat = open("segment-train.txt")
_, meta = arff.loadarff(dat)
dat.close()

dat = open("segment-train.txt")
train_data = arff.loadarff(dat)
dat.close()

df_train = pd.DataFrame(train_data[0])
rf_train = df_train.iloc[:, :19].astype('float')
ac_train = df_train.iloc[:, 19:].astype('str')

dat = open("segment-test.txt")
test_data = arff.loadarff(dat)
dat.close()

df_test = pd.DataFrame(test_data[0])
rf_test = df_test.iloc[:, :19].astype('float')
ac_test = df_test.iloc[:, 19:].astype('str')

nf_train = normalize(rf_train)
nf_test = normalize(rf_test)


def test_dtc_r():
    test_clf(c=lambda r: DecisionTreeClassifier(
        random_state=r))


def test_dtc(min_samples_split=10, max_depth=9, criterion='entropy', splitter='best'):
    test_clf(c=lambda r: DecisionTreeClassifier(
        random_state=r, min_samples_split=min_samples_split, max_depth=max_depth, criterion=criterion, splitter=splitter))

def test_knn(n_neighbors=5):
    test_clf(c=lambda r: KNeighborsClassifier(
        n_neighbors=n_neighbors), rnd=[1])

def test_knn_n(n_neighbors=5):
    test_clf(c=lambda r: KNeighborsClassifier(
        n_neighbors=n_neighbors), rf_train=nf_train, rf_test=nf_test, rnd=[1])


def test_clf(c, rf_train=rf_train, rf_test=rf_test, rnd=[506, 286, 110, 762, 93, 418]):
    for i, rand in enumerate(rnd):
        clf = c(rand)
        clf = clf.fit(rf_train, np.ravel(ac_train))
        rp = clf.predict(rf_test)
        if len(rnd) > 1:
            print("\nRound " + str(i) + " with random_state=" + str(rand))
        print("\nClassification Report of " +
              str(clf))
        print(classification_report(ac_test, rp,
                                    digits=5))


def test_knn_train(n_neighbors=5):
    test_clf_train(c=lambda r: KNeighborsClassifier(
        n_neighbors=n_neighbors), rf_train=nf_train, rf_test=nf_test)


def test_dtc_train():
    test_clf_train(c=lambda r: DecisionTreeClassifier(
        random_state=r))


def test_clf_train(c, rf_train=rf_train, rf_test=rf_test):
    for i, rand in enumerate([506, 286, 110, 762, 93, 418]):
        clf = c(rand)
        clf = clf.fit(rf_train, np.ravel(ac_train))
        rp = clf.predict(rf_train)
        print("\nRound " + str(i) + " with random_state=" + str(rand))
        print("\nClassification Report of " +
              str(clf))
        print(classification_report(ac_train, rp,
                                    digits=5))


def visualize_dtc():
    #   raw_features = rf_test.append(rf_train, ignore_index=True)
    #   actual_classes = ac_test.append(ac_train, ignore_index=True)
    raw_features = rf_train
    actual_classes = ac_train
    clf = DecisionTreeClassifier(min_samples_split=10, max_depth=9,
                                 criterion='entropy', splitter='best', random_state=1247)
    clf = clf.fit(raw_features, np.ravel(actual_classes))
    fns = list(raw_features)
    dot_data = tree.export_graphviz(
        clf, out_file=None, feature_names=fns, filled=True, rounded=True, class_names=meta['class'][1])
    graph = gr.Source(dot_data)
    graph.render("segment_dtc")
